package database;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.XMLEvent;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import application.AppPar;
import application.Cuckoo;
import application.Profile;

import rice.p2p.util.BloomFilter;
import toolbox.HealthIndex;
import toolbox.UTPair;

public class ProfileXMLDB extends XMLDB {

	public ProfileXMLDB(String dir, String path, String root) {
		super(dir, path, root);
	}
	
	public String getNameFromId(String userId) {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					Profile p = getProfile(event, reader);
					if(p.getUserId().compareTo(userId) == 0) {
						in.close();
						return p.getUserName();
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return null;
	}
	
	public synchronized void xmlParsePrint() {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					Profile p = getProfile(event, reader);
					System.out.print(p.toString());
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
	}
	/*
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			
			List<Element> list = root.getChildren();
			for (Element e : list) {
				//System.out.println("User ID:  " + e.getAttributeValue("userId"));
				//System.out.println("Node ID:  " + e.getChildText("nodeId"));
				//System.out.println("Username: " + e.getChildText("username"));
				//System.out.println("Location: " + e.getChildText("location"));
				//System.out.println("Web:      " + e.getChildText("web"));
				//System.out.println("Bio:      " + e.getChildText("bio"));
				//System.out.println("Last modified time: " + e.getChildText("time"));
				System.out.println("xml"+e.getAttributeValue("userId")+" "+e.getChildText("username")+" "+e.getChildText("location")+" "+e.getChildText("web")+" "+e.getChildText("bio")+" "+e.getChildText("time")+" "+e.getChildText("nodeId"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
	}*/
	
	@SuppressWarnings("unchecked")
	public synchronized LinkedList<String> getInfluentialUserIds() {
		LinkedList<String> ll = new LinkedList<String>();
		try {
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					Iterator<Attribute> iterator = event.asStartElement().getAttributes();
					String userId = iterator.next().getValue();
					if(userId == null) {
						System.out.println("in Profile, userId cannot be null!");
						System.exit(0);
					}
					
					for(int i=0; i<3; i++) { //skip from <username> to <web>
						reader.nextEvent(); //<username>
						event = reader.nextEvent();
						if(event.isCharacters()) {
							reader.nextEvent();
						} else if(event.isEndElement()) {}
					}
					
					event = reader.nextEvent();
					if(event.isStartElement() && event.asStartElement().getName().toString().compareTo("bio")==0) {
						event = reader.nextEvent();
						String gon = event.asCharacters().getData();
						if(gon.compareTo("g")==0) {
							ll.add(userId);
						}
					} else {
						System.out.println("error in getInfluentialUserIds()");
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return ll;
	}
	
	public synchronized LinkedList<String> randomSelectHealthyUserIds() {
		//First, select & print to a tmp file
		int num = 0;
		try { 
			PrintWriter os = new PrintWriter(new BufferedWriter(new FileWriter(dir+"/tmp")));
			
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					UTPair utp = getUTPair(event, reader);
					//utp.print();
					//TODO
					if(utp.health > 0) {
						os.println(utp.userId);
						os.flush();
						num ++;
					}
				}
			}
			in.close();
			os.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		System.out.println("num = " + num);
		//Second, do the random selection from the tmp file
		//TODO
		int partnerNum = 2 * (((int) Math.ceil(Math.log(num))) + Cuckoo.C);
		String[] partners = buildPartners(dir+"/tmp", "-1", partnerNum);
		LinkedList<String> ll = new LinkedList<String>();
		for(int i=0; i<partners.length; i++) {
			ll.add(partners[i]);
		}
		return ll;
	}
	
	public static String[] buildPartners(String followeesfollowers, String userId, int partnerNum) {
		int length = getNumOfLine(followeesfollowers);
		int myIndex = getMyIndex(followeesfollowers, userId);
		
		HashSet<Integer> indice = new HashSet<Integer>();
		if(length <= partnerNum) {
			for(int i=0; i<length; i++) {
				indice.add(i); //here you also add itself's index into indice
			}
			indice.remove(myIndex); //remove myself
		} else {
			for(int i=0; i<partnerNum; i++) {
				int idx = (int) (length * Math.random()) - 1;
				if(indice.contains(idx) || idx == myIndex) {
					i--;
				} else {
					indice.add(idx);
				}
			}
		}
		
		if(indice.contains(myIndex) == true) {
			System.out.println("ERROR: CONTAIN MYSELF!!!");
			System.exit(1);
		}
		
		if(indice.size() == 0) {
			return null;
		} else {
			String[] partners = new String[indice.size()];
			try {
				BufferedReader is = new BufferedReader(new FileReader(followeesfollowers));
				String followeeId;
				int p = 0, i = 0;
				while((followeeId = is.readLine()) != null) {
					if(indice.contains(p)) {
						partners[i++] = followeeId.trim();
					}
					p += 1;
				}
				is.close();
			} catch(IOException ioe) {
				ioe.printStackTrace();
			}
			return partners;
		}
	}
	
	public static int getNumOfLine(String filename) {
		int l = 0;
		try {
			BufferedReader is = new BufferedReader(new FileReader(filename));
			while((is.readLine()) != null) {
				l += 1;
			}
			is.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
		return l;
	}
	
	public static int getMyIndex(String filename, String c) {
		int l = 0;
		String line;
		try {
			BufferedReader is = new BufferedReader(new FileReader(filename));
			while((line = is.readLine()) != null) {
				if(line.trim().compareTo(c) == 0) {
					return l;
				}
				l += 1;
			}
			is.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
		return -1;
	}
	
	public synchronized LinkedList<String> getHealthyUserIds() {
		LinkedList<String> uIdList = new LinkedList<String>();

		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					UTPair utp = getUTPair(event, reader);
					if(utp.health > 0) {
						uIdList.add(utp.userId);
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		
		return uIdList;
	}
	/*
	public synchronized LinkedList<Id> getAllNodelIds() {
		LinkedList<Id> idList = new LinkedList<Id>();
		DeterministicNodeIdFactory nidFactory;

		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				nidFactory = new DeterministicNodeIdFactory(e.getAttributeValue("userId"));
				idList.add(nidFactory.generateNodeId());
			}

		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllIds!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllIds!");
		}
		
		return idList;
	}
	*/
	
	public synchronized LinkedList<String> getAllUserIds() {
		LinkedList<String> uIdList = new LinkedList<String>();
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					uIdList.add(getUserId(event, reader));
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return uIdList;
	}
	/*
	public synchronized LinkedList<String> getAllUserIds() {
		LinkedList<String> uIdList = new LinkedList<String>();

		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				uIdList.add(e.getAttributeValue("userId"));
			}

		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllIds!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllIds!");
		}
		
		return uIdList;
	}*/
	
	public synchronized LinkedList<UTPair> getAllUTPair() {
		LinkedList<UTPair> pairList = new LinkedList<UTPair>();
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					UTPair utp = getUTPair(event, reader);
					pairList.add(utp);
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return pairList;
	}
	
	public synchronized UTPair getUTPair(String uid) {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					UTPair utp = getUTPair(event, reader);
					if(utp.userId.compareTo(uid) == 0) {
						in.close();
						return utp;
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return null;
	}
	/*
	public synchronized LinkedList<Profile> getAllProfile() {
		LinkedList<Profile> profileList = new LinkedList<Profile>();
		
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				profileList.addLast(new Profile(e.getAttributeValue("userId"), e.getChildText("username"), e.getChildText("location"), e.getChildText("web"), e.getChildText("bio")));
			}
		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllProfile!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllProfile!");
		}
		
		return profileList;
	}*/

	/*
	public synchronized void addProfile(Twitter.User userProfile) {
		addProfile(new Profile(userProfile));
	}*/
	
	public synchronized void addProfile(Profile profile, String followeefollowerpath) {
		try {
			if(exist(profile.getUserId()) == true) {
				return;
			}
			
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			Element element0 = new Element("userId");
			element0.setAttribute("userId", profile.getUserId());
			
			Element element2 = new Element("username");
			element2.setText(profile.getUserName());

			Element element3 = new Element("location");
			element3.setText(profile.getLocation());

			Element element4 = new Element("web");
			element4.setText(profile.getWeb());

			Element element5 = new Element("bio");
			/****************use this attribute for gossip mark*****************/
			if(followeefollowerpath == null) {
				element5.setText(profile.getBio());
			} else {
				if(getNumOfLine(followeefollowerpath) >= Cuckoo.G_POINT) {
					element5.setText("g");
				} else {
					element5.setText("d");
				}
			}
			
			Element element6 = new Element("time");
			element6.setText("0");
			
			Element element7 = new Element("health");
			element7.setText(String.valueOf(HealthIndex.INIT_BLOOD));
			
			element0.addContent(element2);
			element0.addContent(element3);
			element0.addContent(element4);
			element0.addContent(element5);
			element0.addContent(element6);
			element0.addContent(element7);

			root.addContent(element0);
			doc.setRootElement(root);

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ addProfile in ProfileXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ addProfile in ProfileXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ addProfile in ProfileXMLDB");
		}
	}

	/*
	public synchronized void deleteProfile(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();

			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					root.removeContent(e);
					break;// ?? What should we do?
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			//numOfProfile -= 1;
		} catch(JDOMException jdome) {
			System.out.println("JDOMException in deleteProfile @ ProfileXMLDB"); 
		} catch(IOException ioe) {
			System.out.println("IOException in deleteProfile @ ProfileXMLDB"); 
		}
	}
	

	public synchronized void updateProfile(String userId, Profile profile) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();
			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					e.getChild("username").setText(profile.getUserName());
					e.getChild("location").setText(profile.getLocation());
					e.getChild("web").setText(profile.getWeb());
					e.getChild("bio").setText(profile.getBio());
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ updateProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ updateProfile in ProfileXMLDB");
		}
	}

	public synchronized void updateUserName(String userId, String username)
			throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("username").setText(username);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}

	public synchronized void updateLocation(String userId, String location)
			throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("location").setText(location);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}

	public synchronized void updateWeb(String userId, String web) 
			throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("web").setText(web);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}
	*/
	
	@SuppressWarnings("unchecked")
	public synchronized void updateTime(String userId, String time) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();
			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					e.getChild("time").setText(time);
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException jdome) {
			System.out.println("JDOMException in updateTime() @ ProfileXMLDB");
		} catch(IOException ioe) {
			System.out.println("IOException in updateTime() @ ProfileXMLDB");
		}
	}
	
	public synchronized void updateTime(String userId, long time) {
		updateTime(userId, String.valueOf(time));
	}

	/*
	public synchronized void updateBio(String userId, String bio) throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("bio").setText(bio);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}
	*/
	
//	public int getNumOfProfile() {
//		return numOfProfile;
//	}
	
	public synchronized Profile getProfile(String userId) {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					Profile p = getProfile(event, reader);
					if(p.getUserId().compareTo(userId) == 0) {
						in.close();
						return p;
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return null;
	}
	/*
	public synchronized Profile getProfile(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("userId").compareTo(userId) == 0 ) {
					return new Profile(e.getAttributeValue("userId"), e.getChildText("username"), 
										e.getChildText("location"), e.getChildText("web"), e.getChildText("bio"));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return null;
	}*/
	
	public synchronized long getLastTime(String userid) {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					UTPair utp = getUTPair(event, reader);
					if(utp.userId.compareTo(userid) == 0) {
						in.close();
						return utp.time;
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return -1;
	}

	public synchronized boolean exist(String userId) {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					if(userId.compareTo(getUserId(event, reader)) == 0) {
						in.close();
						return true;
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return false;
	}
	
	/*
	//Whether a user exists in the XML file
	public synchronized boolean exist(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("userId").compareTo(userId) == 0 ) {
					return true;
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return false;
	}*/
	
	/*
	//Init the health index
	//healthIndice is the input parameter
	public synchronized void initHealthIndex(HashMap<String, Integer> healthIndice) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if(Integer.valueOf(e.getChildText("health")) > 0) {
					healthIndice.put(e.getAttributeValue("userId"), Integer.valueOf(e.getChildText("health")));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ initHealthIndex in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ initHealthIndex in ProfileXMLDB");
		}
	}
	*/
	
	@SuppressWarnings("unchecked")
	public synchronized void refreshHealthIndex(HashMap<String, Integer> healthIndice) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				String uid = e.getAttributeValue("userId");
				Integer l = healthIndice.get(uid);
				//System.out.println(l);
				if(l != null) {
					//refresh: add the value
					int h = Integer.valueOf(e.getChildText("health")) + l;
					e.getChild("health").setText(String.valueOf(h));
					/*if( l.compareTo(new Integer(0)) == 0) {
						//filter the dead users in the healthIndice
						healthIndice.remove(uid);
					}*/
				}
			}
			//write to file
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ refreshHealthIndex in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ refreshHealthIndex in ProfileXMLDB");
		}
	}
	
	/*
	public void addNewAttribute(String attrName, int i) {
		addNewAttribute(attrName, String.valueOf(i));
	}
	
	public void addNewAttribute(String attrName, String defaultValue) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			List<Element> list = root.getChildren();
			for (Element e : list) {
				Element newElement = new Element(attrName); 
				newElement.setText(defaultValue);
				e.addContent(newElement);
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ addProfile in ProfileXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ addProfile in ProfileXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ addProfile in ProfileXMLDB");
		}
	}*/
	
	public BloomFilter buildBloomFilter() {
		BloomFilter bloomfilter = new BloomFilter(AppPar.BLOOMFILTER_NUM, AppPar.BLOOMFILTER_LEN);
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					String uid = getUserId(event, reader);
					bloomfilter.add(uid.getBytes());
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return bloomfilter;
		
	}
	/*
	public BloomFilter buildBloomFilter() {
		BloomFilter bloomfilter = new BloomFilter(AppPar.BLOOMFILTER_NUM, AppPar.BLOOMFILTER_LEN);
		
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				String uid = e.getAttributeValue("userId"); 
				bloomfilter.add(uid.getBytes());
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ buildBloomFilter in ProfileXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ buildBloomFilter in ProfileXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ buildBloomFilter in ProfileXMLDB");
		}
		
		return bloomfilter;
	}
	*/
	
	public synchronized int getNumOfEntries() {
		int num = 0;
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("userId")==0) {
					num ++;
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return num;
	}
	/*
	public synchronized int getNumofEntries() {
		int size = 0;
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			if(list == null) {
				size = 0;
			} else {
				size = list.size();
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
		return size;
	}*/
	
	@SuppressWarnings("unchecked")
	private String getUserId(XMLEvent event, XMLEventReader reader) {
		Iterator<Attribute> iterator = event.asStartElement().getAttributes();
		return iterator.next().getValue();
	}
	
	@SuppressWarnings("unchecked")
	private Profile getProfile(XMLEvent firstEvent, XMLEventReader reader) {
		String userId = null; String username = null; String location = null; String web = null; String bio = null;
		try {
			XMLEvent event = firstEvent;
			Iterator<Attribute> iterator = event.asStartElement().getAttributes();
			userId = iterator.next().getValue();
			
			if(userId == null) {
				System.out.println("in Profile, userId cannot be null!");
				System.exit(0);
			}
			
			reader.nextEvent(); //<username>
			event = reader.nextEvent();
			if(event.isCharacters()) {
				username = event.asCharacters().getData();
				//System.out.println("username: " + username);
				reader.nextEvent();
			} else if(event.isEndElement()) {username = "";}
			
			reader.nextEvent(); //<location>
			event = reader.nextEvent();
			if(event.isCharacters()) {
				location = event.asCharacters().getData();
				//System.out.println("location: " + location);
				reader.nextEvent();
			} else if(event.isEndElement()) {location = "";}
			
			reader.nextEvent(); //<web>
			event = reader.nextEvent();
			if(event.isCharacters()) {
				web = event.asCharacters().getData();
				//System.out.println("web: " + web);
				reader.nextEvent();
			} else if(event.isEndElement()) {web = "";}
			
			reader.nextEvent(); //<bio>
			event = reader.nextEvent();
			if(event.isCharacters()) {
				bio = event.asCharacters().getData();
				//System.out.println("bio: " + bio);
				reader.nextEvent();
			} else if(event.isEndElement()) {bio = "";}
			
		}catch (Exception e) {  
			e.printStackTrace();  
		}
		return new Profile(userId, username, location, web, bio);
	}
	
	@SuppressWarnings("unchecked")
	private UTPair getUTPair(XMLEvent firstEvent, XMLEventReader reader) {
		String userId = null; long time = 0; int health = 0;
		try {
			XMLEvent event = firstEvent;
			Iterator<Attribute> iterator = event.asStartElement().getAttributes();
			userId = iterator.next().getValue();
			
			if(userId == null) {
				System.out.println("in Profile, userId cannot be null!");
				System.exit(0);
			}
			
			for(int i=0; i<4; i++) { //skip from <username> to <bio>
				reader.nextEvent(); //<username>
				event = reader.nextEvent();
				if(event.isCharacters()) {
					reader.nextEvent();
				} else if(event.isEndElement()) {}
			}
			
			reader.nextEvent(); //<web>
			event = reader.nextEvent();
			try{
				time = Long.valueOf(event.asCharacters().getData());
			} catch(NumberFormatException nfe) {
				//System.out.println("-------" + event.asCharacters().getData());
			}
			//System.out.print(time + " ");
			reader.nextEvent();
			
			reader.nextEvent(); //<health>
			event = reader.nextEvent();
			health = Integer.valueOf(event.asCharacters().getData());
			//System.out.println(health + " ");
			reader.nextEvent();
		}catch (Exception e) {  
			e.printStackTrace();  
		}
		return new UTPair(userId, time, health);
	}
	
	static public void main(String ars[]) throws JDOMException, IOException {
		
		ProfileXMLDB xmlDB = new ProfileXMLDB("xml/16918503", "xml/16918503/followeeprofile.xml", "followeeprofile");
		
		//ProfileXMLDB test = new ProfileXMLDB("xml", "xml/followeeprofile.xml", "followeeprofile");
		//xmlDB.addProfile(new Profile("456", "Tianyin", "Goettingen",
		//		"http://cs.nju.edu.cn/dislab/xuty/xuty.html", "Hello!"));
		//xmlDB.xmlParsePrint();
		//xmlDB.updateProfile("456", new Profile("456", "234", "Tianyin", "Nanjing",
		//		"http://cs.nju.edu.cn/dislab/xuty/xuty.html", "Hello!"));
		//xmlDB.xmlParsePrint2();
		//System.out.println(xmlDB.getProfile("14610455"));
		//xmlDB.updateBio("456", "I'm enjoying the coding!!!");
		//xmlDB.updateTime("456", 12345);
		
		//xmlDB.xmlParsePrint();
		//System.out.println(xmlDB.getUTPair("89260471"));
		//System.out.println(xmlDB.getAllUTPair().size());
		
		//UTPair.printUTPairList(xmlDB.getAllUTPair());
		
		//Profile p = xmlDB.getProfile("123");
		//p.print();
		
		//System.out.println(xmlDB.getNumFromId("456"));
		//xmlDB.getProfile("456").print();
		//System.out.println(xmlDB.getNumofEntries());
		//System.out.println(xmlDB.exist("50941935"));
		
		/*List<String> tids = test.getAllUserIds();
		BloomFilter bf = xmlDB.buildBloomFilter();
		Iterator<String> iterator = tids.iterator();
		while(iterator.hasNext()) {
			String s = iterator.next();
			if(bf.check(s.getBytes()) != xmlDB.exist(s)) {
				System.out.println(s);
			}
		}*/
		
		/*List<String> l = xmlDB.getHealthyUserIds();
		Iterator<String> iterator = l.iterator();
		while(iterator.hasNext()) {
			String i = iterator.next();
			System.out.print(i + " ");
			System.out.println(xmlDB.getNameFromId(i)+" ");
			//System.out.println(xmlDB.getLastTime(i));
		}*/
		
		/*List<String> l = xmlDB.randomSelectHealthyUserIds();
		Iterator<String> iterator = l.iterator();
		while(iterator.hasNext()) {
			String i = iterator.next();
			System.out.print(i + " ");
			//System.out.println(xmlDB.getNameFromId(i)+" ");
			//System.out.println(xmlDB.getLastTime(i));
		}*/
		
		List<String> l = xmlDB.getInfluentialUserIds();
		Iterator<String> iterator = l.iterator();
		while(iterator.hasNext()) {
			String i = iterator.next();
			System.out.print(i + " ");
			System.out.println(xmlDB.getNameFromId(i)+" ");
			//System.out.println(xmlDB.getLastTime(i));
		}
	}

}
