package emulascript;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

import application.Cuckoo;
import application.Profile;
import database.PartnerXMLDB;
import database.ProfileXMLDB;

public class UserDataGenerator {
	public static final String IDLIST = "./IdList";
	public static final String ALLDIR = "D:/TwTest/"; //raw data
	//public static final String USERDATA = "D:/UserData/"; //user's xml-based data
	public static final String USERNPORT = "./userNport";
	public static final String EMU_DATA = "D:/EmulationData/"; //emulation data
	
	//make dirs for each user
	public void mkdirs() {
		try {
			//first make the outter side dir
			File outdir = new File(EMU_DATA);
			if(outdir.exists() == false) {
				outdir.mkdir();
			}
			
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				userId = userId.trim();
				File idfile = new File(EMU_DATA+userId);
				idfile.mkdir();
				File xifile = new File(EMU_DATA+userId+"/xml/");
				xifile.mkdir();
				File iifile = new File(EMU_DATA+userId+"/xml/"+userId);
				iifile.mkdir();
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void buildPortFile() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(USERNPORT));
			String line = null;
			while((line = is.readLine()) != null) {
				line = line.trim();
				String[] split = line.split("#");
				String userId = split[0];
				PrintWriter os = new PrintWriter(new BufferedWriter(new FileWriter(EMU_DATA+userId+"/xml/"+userId+"/"+userId+"_port")));
				os.println(line);
				os.flush();
				os.close();
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	//Here we do not care the username, web, location & bio
	public void buildXMLProfiles() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				userId = userId.trim();
				String dir = EMU_DATA+userId+"/xml/"+userId;
				ProfileXMLDB myProfileDB = new ProfileXMLDB(dir, dir + "/myprofile.xml", "myprofile");
				myProfileDB.addProfile(new Profile(userId, "n", "n", "n", "n"), null);
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	//Here we build the followee/follower xml-based db.
	//Still, we just care the userId
	public void buildFolloweeLinks() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				String dir = EMU_DATA+userId+"/xml/"+userId;
				
				//if exist, delete
				File file = new File(dir + "/followeeprofile.xml");
				if(file.exists()) {file.delete();}
				
				ProfileXMLDB followeeProfileDB = new ProfileXMLDB(dir, dir + "/followeeprofile.xml", "followeeprofile");
				try {
					BufferedReader fis = new BufferedReader(new FileReader(ALLDIR+userId+"/"+userId+"_followee"));
					String followeeId;
					while((followeeId = fis.readLine()) != null) {
						followeeId = followeeId.trim();
						followeeProfileDB.addProfile(new Profile(followeeId, "n", "n", "n", "n"), null);
					}
					fis.close();
				} catch(IOException ioe) {
					ioe.printStackTrace();
				}
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void buildFollowerLinks() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				String dir = EMU_DATA+userId+"/xml/"+userId;
				
				//if exist, delete
				File file = new File(dir + "/followerprofile.xml");
				if(file.exists()) {file.delete();}
				
				ProfileXMLDB followerProfileDB = new ProfileXMLDB(dir, dir + "/followerprofile.xml", "followerprofile");
				try {
					BufferedReader fis = new BufferedReader(new FileReader(ALLDIR+userId+"/"+userId+"_follower"));
					String followerId;
					while((followerId = fis.readLine()) != null) {
						followerId = followerId.trim();
						followerProfileDB.addProfile(new Profile(followerId, "n", "n", "n", "n"), null);
					}
					fis.close();
				} catch(IOException ioe) {
					ioe.printStackTrace();
				}
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void buildPartners() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				String dir = EMU_DATA+userId+"/xml/"+userId;
				
				//if exist, delete
				File file = new File(dir + "/partner.xml");
				if(file.exists()) {file.delete();}
				
				PartnerXMLDB partnerXMLDB = new PartnerXMLDB(dir, dir + "/partner.xml", "partner");
				ProfileXMLDB followeeProfileDB = new ProfileXMLDB(dir, dir + "/followeeprofile.xml", "followeeprofile");
				LinkedList<String> followees = followeeProfileDB.getAllUserIds();
				Iterator<String> iterator = followees.iterator();
				while(iterator.hasNext()) {
					String fid = iterator.next();
					String followeesfollowers = ALLDIR + fid + "/" + fid + "_follower";
					//TODO
					int partnerNum = ((int) Math.ceil(Math.log(ProfileXMLDB.getNumOfLine(followeesfollowers)))) + Cuckoo.C;
					String[] partners = ProfileXMLDB.buildPartners(followeesfollowers, fid, partnerNum);
					partnerXMLDB.initWrite(partners, fid);
				}
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void testSet() {
		HashSet<Integer> set = new HashSet<Integer>();
		set.add(1);
		System.out.println(set.contains(1));
	}
	
	public void testBuildPartners() {
		String file = "./TestBuildPartners";
		String uid = "5";
		/*
		try {
			PrintWriter os = new PrintWriter(new BufferedWriter(new FileWriter(file)));
			for(int i=0; i<100; i++) {
				os.println(i);
				os.flush();
			}
			os.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		*/
		String[] partners = ProfileXMLDB.buildPartners(file, uid, 10);
		for(int i=0; i<partners.length; i++) {
			System.out.println(partners[i]);
		}
	}
	
	/*
	//At the first run: via building xml`
	public void buildSocialLinks(String uid) {
		String dir = "xml/" + uid;
		
		//build followees
		ProfileXMLDB followeeProfileDB = new ProfileXMLDB(dir, dir + "/followeeprofile.xml", "followeeprofile");
		try {
			BufferedReader is = new BufferedReader(new FileReader(FOLLOWEES));
			String followeeId;
			while((followeeId = is.readLine()) != null) {
				followeeId = followeeId.trim();
				followeeProfileDB.addProfile(new Profile(followeeId, null, null, null, null));
			}
			is.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
		
		//build followers
		ProfileXMLDB followerProfileDB = new ProfileXMLDB(dir, dir + "/followerprofile.xml", "followerprofile");
		try {
			BufferedReader is = new BufferedReader(new FileReader(FOLLOWERS));
			String followerId;
			while((followerId = is.readLine()) != null) {
				followerId = followerId.trim();
				followerProfileDB.addProfile(new Profile(followerId, null, null, null, null));
			}
			is.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
		
		//build partners list
		PartnerXMLDB partnerXMLDB = new PartnerXMLDB(dir, dir + "/partner.xml", "partner");
		try {
			BufferedReader is = new BufferedReader(new FileReader(FOLLOWEES));
			String followeeId;
			while((followeeId = is.readLine()) != null) {
				followeeId = followeeId.trim();
				String followeesfollowers = ALL_FOLLOWERS + followeeId + "_followers";
				//TODO
				//int partnerNum = 10;//getPartnerNum, should be O(log n);
				String[] partners = buildPartners(followeesfollowers, uid, 10);
				partnerXMLDB.initWrite(partners, uid);
			}
			is.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}
	*/
	
	public void shiftTweets() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				userId = userId.trim();
				String ffile = ALLDIR + userId + "/" + userId + "_newtweets";
				String tofile = EMU_DATA+userId+"/xml/"+userId + "/" +userId + "_tweet";
				copyfile(ffile, tofile);
			}
			is.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void shiftJars() {
		int i = 0;
		try {
			BufferedReader is = new BufferedReader(new FileReader(IDLIST));
			String userId = null;
			while((userId = is.readLine()) != null) {
				userId = userId.trim();
				String ffile = "./CuckooLight.jar";
				String tofile = EMU_DATA+userId+"/CuckooLight.jar";
				File file = new File(tofile);
				if(file.exists()) {file.delete();}
				copyfile(ffile, tofile);
				i++;
			}
			is.close();
			System.out.println("i = " + i);
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void copyfile(String srFile, String dtFile) {
		try {
			File f1 = new File(srFile);
		    File f2 = new File(dtFile);
		    InputStream in = new FileInputStream(f1);
		      
		    //For Append the file.
		    //OutputStream out = new FileOutputStream(f2,true);

		    //For Overwrite the file.
		    OutputStream out = new FileOutputStream(f2);

		    byte[] buf = new byte[1024];
		    int len;
		    while ((len = in.read(buf)) > 0){
		    	out.write(buf, 0, len);
		    }
		    out.flush();
		    out.close();
		    in.close();
		    //System.out.println("File copied.");
		    
		} catch(FileNotFoundException ex){
		    System.out.println(ex.getMessage() + " in the specified directory.");
		    try {
		    	new File(dtFile).createNewFile();
		    } catch(Exception e) {
		    	e.printStackTrace();
		    }
		} catch(IOException e){
		    System.out.println(e.getMessage());      
		}
	}
	
	public static void main(String[] args) {
		UserDataGenerator dg = new UserDataGenerator();
		//dg.printIdList2File();
		//dg.mkdirs();
		//dg.buildXMLProfiles();
		//dg.buildFolloweeLinks();
		//dg.buildFollowerLinks();
		//dg.buildPartners();
		dg.buildPortFile();
		//dg.shiftTweets();
		//dg.shiftJars();
	}
	
}
