package message;
import java.io.IOException;

import application.Status;

import rice.p2p.commonapi.Endpoint;
import rice.p2p.commonapi.Id;
import rice.p2p.commonapi.NodeHandle;
import rice.p2p.commonapi.rawserialization.*;

public class RplStatusMsg extends CuckooMsg {
	private static final long serialVersionUID = 201004221851L;
	public static final short TYPE = 12;
    
    private String authorId;
    
    private String[] stid; //statusId
    private String[] text;
    private long[] time; //The text's publishing time;
    
    //FOR MEASUREMENTS -- piggybacked by the ReqStatusMsg
    private int achops; 
    private long actime;
    private int rehops;
    private long retime;
    
    // Constructors.   
    public RplStatusMsg(Id from, Id to, String fid, String tid, NodeHandle nh, long t, String authorId, Status[] status, int ach, long act, int reh, long ret) {
    	super(from, to, fid, tid, nh, t);
    	
    	this.authorId = authorId;
    
    	if(status != null) {
    		int l = status.length;
    		stid = new String[status.length];
    		text = new String[status.length];
    		time = new long[status.length];
    	
    		for(int i=0; i<l; i++) {
    			stid[i] = status[i].getStatusId();
    			text[i] = status[i].getText();
    			time[i] = status[i].getTime();
    		}
    	} else {
    		stid = null;
    		text = null;
    		time = null;
    	}
    	
    	achops = ach;
    	actime = act;
    	rehops = reh;
    	retime = ret;
    }
    
	public String toString() {
    	return "CuckooMsg: " + text + "(@" + time + ")" +  " ( from " + from + " to " + to;
    }

	public short getType() {
		return TYPE;
	}

	public void serialize(OutputBuffer buf) throws IOException {
		super.serialize(buf);
		
	    buf.writeUTF(authorId);
	    
	    buf.writeInt(achops);
	    buf.writeLong(actime);
	    buf.writeInt(rehops);
	    buf.writeLong(retime);
	    
	    //Just use readUTF
	    if(text != null) {
	    	buf.writeInt(text.length);
	    	for(int i = 0; i < text.length; i++) {
	    		buf.writeUTF(stid[i]);
	    		buf.writeUTF(text[i]);
	    		buf.writeLong(time[i]);
	    	}
	    } else {
	    	buf.writeInt(0);
	    }
	}

	public RplStatusMsg(InputBuffer buf, Endpoint endpoint) throws IOException {
		super(buf, endpoint);
	    authorId = buf.readUTF();
	    
	    achops = buf.readInt();
	    actime = buf.readLong();
	    rehops = buf.readInt();
	    retime = buf.readLong();
	    
	    int l = buf.readInt();
	    if(l == 0) {
	    	stid = null;
	    	text = null;
	    	time = null;
	    } else {
	    	stid = new String[l];
	    	text = new String[l];
	    	time = new long[l];
	    	for(int i=0; i<l; i++) {
	    		stid[i] = buf.readUTF();
	    		text[i] = buf.readUTF();
	    		time[i] = buf.readLong();
	    	}
	    }
	}
	
	public String[] getRecentStids() {
		return stid;
	}
	
	public String[] getRecentTexts() {
		return text;
	}
	
	public long[] getRecentTimes() {
		return time;
	}
	
	public String getAuthorId() {
		return authorId;
	}
	
	public int getContentLength() {
		int sum = 0;
		if(text == null) {
			return 0;
		} else {
			for(int i=0; i<text.length; i++) {
				sum += text[i].length();
			}
			return sum;
		}
	}
	
	public int getAcHops() {
		return achops;
	}
	
	public long getAcTime() {
		return actime;
	}
	
	public int getReHops() {
		return rehops;
	}
	
	public long getReTime() {
		return retime;
	}
}
